Crate literate_crypto
source ·Expand description
Modules
- AES is a commonly used block cipher.
- Elliptic curve cryptography.
- SHA-1 and SHA-2 are hash functions specified by FIPS 180-4.
- SHA-3 is a modern hash function specified by FIPS 202.
Structs
- AES block cipher with 128-bit keys.
- AES block cipher with 192-bit keys.
- AES block cipher with 256-bit keys.
- Cipher block chaining mode is the most common mode of operation for block ciphers which combines every block with all of the previous blocks of ciphertext.
- Block counter mode is a block chaining mode which turns a block cipher into a stream cipher, and hence does not require a padding scheme.
- The Davies-Meyer construction turns a block cipher into a Merkle-Damgard compression function.
- Electronic codebook mode, a simple and insecure mode of operation.
- Elliptic curve digital signature algorithm.
- A hash-based message authentication code is a method for creating a MAC from a hash function.
- Error indicating that a private key is invalid.
- Error indicating that a signature is invalid.
- The Merkle-Damgard construction, used as a building block for hash functions.
- A multisig scheme based on Schnorr signatures.
- The one-time pad is a simple cipher which XORs the input plaintext with the key.
- PKCS #7, a simple approach to padding.
- Schnorr is a simple, efficient, and provably secure (under the ECDLP assumption) signature algorithm.
- Before creating a Schnorr multisig, the actors must each commit to a secret random number $r_i$. They proceed in two rounds:
- Spontaneous anonymous group signatures.
- SHA-1 hash specified by FIPS 180-4.
- SHA-3 hash with 224-bit output.
- SHA-3 hash with 256-bit output.
- SHA-3 hash with 384-bit output.
- SHA-3 hash with 512-bit output.
- SHA-224 hash specified by FIPS 180-4.
- SHA-256 hash specified by FIPS 180-4.
Traits
- A block cipher encrypts and decrypts data in blocks of fixed size.
- The decryption half of a block cipher.
- The encryption half of a block cipher.
- A way to execute a block cipher on data of arbitrary length.
- A cipher encrypts and decrypts data of arbitrary length using a symmetric key.
- The decryption half of a cipher.
- The encryption half of a cipher.
- A compression function used internally by the Merkle-Damgard construction.
- Cryptographically secure pseudorandom number generator.
- The step function of the Davies-Meyer construction defines how the previous hash state is combined with the new hash state.
- A source of entropy.
- A cryptographic hash function maps a preimage of arbitrary length into a fixed-size hash digest.
- A message authentication code algorithm is a method for computing a keyed hash.
- The padding scheme used by the Merkle-Damgard construction.
- A multisig scheme is similar to a regular signature, except that it is signed by multiple private keys and verified with multiple public keys.
- A scheme to pad messages to be a multiple of some block size.
- Ring signature scheme.
- A signature scheme is a method by which an actor proves that he generated a message.
Functions
- Randomly shuffle the elements of a slice.
- Draw a uniformly random number from a range.