pub struct LengthPadding(/* private fields */);Expand description
SHA-2 length padding.
The preimage is padded by appending a single 1 bit, followed by as many bits as needed to pad to a multiple of 512 - 64 = 448 bits, followed by the bit length of the preimage encoded as an unsigned big-endian 64 bit integer. This results in a Merkle-Damgard compliant padding into blocks of 512 bits.